/////////////////////////////////////////////////////////////////////////////
// PolygonDlg.cpp defines the plugin dialog implementation

/////////////////////////////////////////////////////////////////////////////
// QCKPLUGIN API
// Plugin header files

#include "stdafx.h"
#include "PolygonDlg.h"
#include <math.h>

/////////////////////////////////////////////////////////////////////////////
// QCKPLUGIN API
// sQCKQUERYSETTINGS constructor

sQCKQUERYSETTINGS::sQCKQUERYSETTINGS() {}

/////////////////////////////////////////////////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// QCKPLUGIN API
// Initialize plugin dialog (one time)

BOOL CPolygonDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

// Register plugin services

	m_GraphicsSrvc = m_Manager->RegisterForGraphicsService();
  m_WindowSrvc = m_Manager->RegisterForWindowService();
  m_QuerySrvc = m_Manager->RegisterForQueryService();

// Initialize members

  m_Drawing = false;
  m_Filename = "";
  m_File = NULL;

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// QCKPLUGIN API
// Enable/disable service notifications when dialog is displayed/hidden

void CPolygonDlg::OnShowWindow(BOOL bShow, UINT nStatus) 
{
	CDialog::OnShowWindow(bShow, nStatus);
	
// Being shown

  if(bShow) {

// Switch draw notification on

    m_GraphicsSrvc->SetDrawServiceNotify(m_Handle, this);

// Draw the current selection window

    DrawWindow();

  } else {

// Being hidden

// Switch notifications off

    m_GraphicsSrvc->SetDrawServiceNotify(m_Handle, NULL);
    m_WindowSrvc->SetDrawRubberBand(false);
    m_Drawing = false;
  }
}

/////////////////////////////////////////////////////////////////////////////
// QCKPLUGIN API
// Show dialog notification used to hide the plugin dialog during redraw

void CPolygonDlg::OnShowDialog(const bool show) 
{

// Move dialog on/off screen during redraw

  CRect rect;
  GetWindowRect(&rect);
  if(show) {
    rect.left-=3000;
    rect.right-=3000;
  } else {
    rect.left+= 3000;
    rect.right+= 3000;
  }
  MoveWindow(&rect);
}

/////////////////////////////////////////////////////////////////////////////
// QCKPLUGIN API
// Select a window in the Qckvu3 viewer

void CPolygonDlg::OnSelect() 
{

// Switch window notification on

  m_WindowSrvc->SetWindowServiceNotify(m_Handle, this);
  m_WindowSrvc->SetDrawRubberBand(true);
  m_Drawing = true;

// Erase old selection window

  m_GraphicsSrvc->Redraw(false);  
}

/////////////////////////////////////////////////////////////////////////////
// QCKPLUGIN API
// Rubber band notidication from Qckvu3

void CPolygonDlg::OnDrawWindowRubberBand(const sQCKWINDOW& window)
{   

// Display window selection in the dialog

  m_X1 = window.mMinX;
  m_Y1 = window.mMinY;
  m_X2 = window.mMaxX;
  m_Y2 = window.mMaxY;
  m_Window.Format("(%.5f, %.5f) - (%.5f, %.5f)", m_X1, m_Y1, m_X2, m_Y2);
  UpdateData(FALSE);

// Switch window notification off

  m_WindowSrvc->SetWindowServiceNotify(m_Handle, NULL);
  m_WindowSrvc->SetDrawRubberBand(false);
  m_Drawing = false;

// Draw the selection window

  DrawWindow();
}   

/////////////////////////////////////////////////////////////////////////////
// QCKPLUGIN API
// Draw the selection window on top of the Qckvu3 viewer

void CPolygonDlg::DrawWindow()
{

// Build a rectangle from the window co-ordinates

  double p[10];
 
  p[0] = m_X1;
  p[1] = m_Y1;
  p[2] = m_X2;
  p[3] = m_Y1;
  p[4] = m_X2;
  p[5] = m_Y2;
  p[6] = m_X1;
  p[7] = m_Y2;
  p[8] = p[0];
  p[9] = p[1];

// Don't draw a zero-sized widnow

  if(fabs(p[0] - p[4]) < 1e-12 || fabs(p[1] - p[5]) < 1e-12)
    return;

// Set the drawing color to black

  m_GraphicsSrvc->SelectColor(0, 0, 0);

// Set the line thickness to 3 pixels

  m_GraphicsSrvc->SetLineSize(3);

// Draw the selection window

  m_GraphicsSrvc->DrawBoundary(NQckPlugin::OUTLINE, p, 5, true);
}


/////////////////////////////////////////////////////////////////////////////
// QCKPLUGIN API
// Redraw notification from Qckvu

void CPolygonDlg::OnRedraw()
{

// If selection is not taking place, redraw the selection window

  if(!m_Drawing)
    DrawWindow();
}

/////////////////////////////////////////////////////////////////////////////
// QCKPLUGIN API
// Cancel button press event

void CPolygonDlg::OnCancel() 
{
  ShowWindow(SW_HIDE);
	CDialog::OnCancel();

// Erase the selection window

  m_GraphicsSrvc->Redraw(false);	
}

/////////////////////////////////////////////////////////////////////////////
// QCKPLUGIN API
// Browse button press event

void CPolygonDlg::OnBrowse() 
{
  UpdateData(TRUE);

// Define available file types

  const char *ft = "Text Files (*.txt)|*.txt|All Files (*.*)|*.*|";

// Create file dialog

  CFileDialog dlg(FALSE, ".txt", "", OFN_HIDEREADONLY, ft, NULL);
  dlg.m_ofn.lpstrTitle = "Export Polygon FIle";

// Display dialog

  if(dlg.DoModal() == IDOK) {

// If a new file name is selected, displya it in the dialog

    m_Filename = dlg.GetPathName();	
    UpdateData(FALSE);
  }
}

/////////////////////////////////////////////////////////////////////////////
// QCKPLUGIN API
// OK button press event

void CPolygonDlg::OnOK() 
{

// Check the file name

  if(strcmp(m_Filename, "") == 0) {
    AfxMessageBox("Please select a file name.");
    return;
  }

// Open the file for writing

  m_File = fopen(m_Filename, "w");
  if(m_File == NULL) {
    AfxMessageBox("The file name is not valid.");
    return;
  }

// Define settinga to retrieve vector data from Qckvu3
  
  sQCKWINDOW window;
  window.mMinX = m_X1;
  window.mMinY = m_Y1;
  window.mMaxX = m_X2;
  window.mMaxY = m_Y2;

  sQCKQUERYSETTINGS settings;
  settings.mGetText = false;
  settings.mWindow = window;

// Retrieve vector data

  m_QuerySrvc->GetDataVector(this, &settings);

// Close the file

  fclose(m_File);	

// Prompt to view the output file

  int answer = AfxMessageBox("Would you like to view the output file?", MB_ICONINFORMATION | MB_YESNO);
  if(answer == IDYES) {
    ShellExecute(NULL, NULL, m_Filename, NULL, NULL, SW_SHOWNORMAL);	
  }

  ShowWindow(SW_HIDE);
	CDialog::OnOK();

// Erase the selection window

  m_GraphicsSrvc->Redraw(false);

}

/////////////////////////////////////////////////////////////////////////////
// QCKPLUGIN API
// Get data vector notification from Qckvu

void CPolygonDlg::OnGetDataVector(const sQCKVECTOR& VectorDatum)
{

// Process boundary data

  if(VectorDatum.mType == 'B') {
    sQCKBOUNDARY boundary = VectorDatum.mBoundary;

// Write cell and layer data to disk

    fprintf(m_File, "POLYGON\n");
    fprintf(m_File, "Cell: %s\n", boundary.mCellName);
    fprintf(m_File, "Layer: %d:%d\n", boundary.mLayer, boundary.mDataType);

// Get co-ordinate data

    int n = boundary.mNumVert;
    const double *xy = boundary.mXY;

// Write co-ordinate data to disk

    fprintf(m_File, "Points: %d\n", n);
    int i = 0;
    for(i = 0; i < n; i++) {
      fprintf(m_File, "%f %f\n", xy[i * 2], xy[i * 2 + 1]);
    }
    fprintf(m_File, "\n");
  }

// Process path data

  if(VectorDatum.mType == 'P') {
    sQCKPATH path = VectorDatum.mPath;

// Write cell and layer data to disk

    fprintf(m_File, "PATH\n");
    fprintf(m_File, "Cell: %s\n", path.mCellName);
    fprintf(m_File, "Layer: %d:%d\n", path.mLayer, path.mDataType);

// Write path width to disk

    fprintf(m_File, "Width: %f\n", path.mWidth);

// Get co-ordinate data

    int n = path.mNumVert;
    const double *xy = path.mXY;

// Write co-ordinate data to disk

    fprintf(m_File, "Points: %d\n", n);
    int i = 0;
    for(i = 0; i < n; i++) {
      fprintf(m_File, "%f %f\n", xy[i * 2], xy[i * 2 + 1]);
    }
    fprintf(m_File, "\n");
  }

}

/////////////////////////////////////////////////////////////////////////////
// // AppWizard generated content

CPolygonDlg::CPolygonDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CPolygonDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CPolygonDlg)
	m_Window = _T("");
	m_Filename = _T("");
	//}}AFX_DATA_INIT
}

void CPolygonDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPolygonDlg)
	DDX_Text(pDX, IDC_WINDOW, m_Window);
	DDX_Text(pDX, IDC_FILENAME, m_Filename);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CPolygonDlg, CDialog)
	//{{AFX_MSG_MAP(CPolygonDlg)
	ON_WM_SHOWWINDOW()
	ON_BN_CLICKED(ID_SELECT, OnSelect)
	ON_BN_CLICKED(ID_BROWSE, OnBrowse)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////



